--[[
--  thek_Unitframes
--  version: 1.1.cataclysm
--  author:  thek
--]]


local c, uf, a, b, color = thek_Unitframes_Config, {}, CreateFrame("Frame", "thek_Unitframes", UIParent), {}, nil;

-- go --------------------------------------------------------------------
local uf    = {};
    uf["player"] = "PlayerFrame";
    uf["target"] = "TargetFrame";
    uf["focus"]  = "FocusFrame";

-- create frames
for v, _ in pairs(uf) do
    a[v] = CreateFrame("Frame", "thek_Unitframes_"..v, _G[uf[v]]);
    a[v]:SetWidth(_G[uf[v]]:GetWidth());
    a[v]:SetHeight(_G[uf[v]]:GetHeight());
    if v ~= "focus" then
        a[v]:SetPoint(unpack({_G[uf[v]]:GetPoint()}));
        a[v]:SetClampedToScreen(true);
        a[v]:SetMovable(true)
        a[v]:RegisterForDrag("LeftButton");
        a[v]:EnableMouse(false);    
        a[v]:SetScript("OnDragStart", function(self) self:StartMoving() end);
        a[v]:SetScript("OnDragStop", function(self) self:StopMovingOrSizing() end);
        a[v].name = a[v]:CreateFontString(nil, "OVERLAY");
        a[v].name:SetFont(c["font"].face, c["font"].size, c["font"].options);
        a[v].name:SetPoint("TOP", a[v], 0, 5);
        a[v].locked = true;
    else
        a[v]:SetPoint("TOPLEFT", _G[uf[v]], "TOPLEFT");
    end;
    -- add health percent if enabled
    if c[v].percent[1] then
        a[v].percent = a[v]:CreateFontString(nil, "ARTWORK");
        a[v].percent:SetFont(c["font"].face, c["font"].size, c["font"].options);
        a[v].percent:SetPoint(c[v].percent[2], a[v], c[v].percent[3], c[v].percent[4], c[v].percent[5]);
        a[v].percent:SetJustifyH("RIGHT");
    end;
    if c[v].current[1] then
        a[v].current = a[v]:CreateFontString(nil, "ARTWORK");
        a[v].current:SetFont(c["font"].face, c["font"].size, c["font"].options);
        a[v].current:SetPoint(c[v].current[2], a[v], c[v].current[3], c[v].current[4], c[v].current[5]);
        a[v].current:SetJustifyH("RIGHT");
        if v == "target" then 
            TargetFrameTextureFrameHealthBarText.Show = function() end
        elseif v == "player" then 
            PlayerFrameHealthBarText.Show = function() end;
        end;
    end;
end;

-- classicon portraits
if c.showClassIcons then
    hooksecurefunc("UnitFramePortrait_Update", function(self)
        if self.portrait then
            if self.unit == "player" or self.unit == "pet" or self.unit == "vehicle" then return end;
            local t = CLASS_ICON_TCOORDS[select(2, UnitClass(self.unit))];
            if t then 
                self.portrait:SetTexture("Interface\\TargetingFrame\\UI-Classes-Circles");
                self.portrait:SetTexCoord(unpack(t));
            end;
        end;
    end);
end;

-- get healthtext
local function GetHealthValues(unit)
    local hp = UnitHealth(unit);
    if hp > 0 then
        if c[unit].current[1] then
            if hp > 1000 then 
                a[unit].current:SetFormattedText("%.1fk", (hp / 1000));
            else
                a[unit].current:SetText(hp);
            end;
        end;
        if c[unit].percent[1] then
            hp = hp / UnitHealthMax(unit) * 100;
            a[unit].percent:SetFormattedText("%.1f%%", hp);
        end
    else
        a[unit].current:SetText("");
        a[unit].percent:SetText("");
    end;
end;

-- movable unitframes
local function SetUnitframe(unit)
    if InCombatLockdown() or not unit then return end;
    _G[uf[unit]]:ClearAllPoints();
    _G[uf[unit]]:SetPoint("TOPLEFT", a[unit], "TOPLEFT");
end;

-- hooks for movable unitframes
local function SetPlayerFrameAfterVehicleCrap()
    if InCombatLockdown() then return end;
    SetUnitframe("player");
end;
hooksecurefunc("PlayerFrame_UpdateArt",        SetPlayerFrameAfterVehicleCrap);
hooksecurefunc("PlayerFrame_SequenceFinished", SetPlayerFrameAfterVehicleCrap);

-- hooks for targetauras
local function UpdateTargetAuraPositions(self, auraName, numAuras, numOppositeAuras, largeAuraList, updateFunc, maxRowWidth, offsetX)
    local AURA_OFFSET_Y = 3;
    local LARGE_AURA_SIZE = c["targetaura"].largeSize;
    local SMALL_AURA_SIZE = c["targetaura"].smallSize;
    local size;
    local offsetY = AURA_OFFSET_Y;
    local rowWidth = 0;
    local firstBuffOnRow = 1;
    for i=1, numAuras do
        if ( largeAuraList[i] ) then
            size = LARGE_AURA_SIZE;
            offsetY = AURA_OFFSET_Y + AURA_OFFSET_Y;
        else
            size = SMALL_AURA_SIZE;
        end
        if ( i == 1 ) then
            rowWidth = size;
            self.auraRows = self.auraRows + 1;
        else
            rowWidth = rowWidth + size + offsetX;
        end
        if ( rowWidth > maxRowWidth ) then
            updateFunc(self, auraName, i, numOppositeAuras, firstBuffOnRow, size, offsetX, offsetY);
            rowWidth = size;
            self.auraRows = self.auraRows + 1;
            firstBuffOnRow = i;
            offsetY = AURA_OFFSET_Y;
        else
            updateFunc(self, auraName, i, numOppositeAuras, i - 1, size, offsetX, offsetY);
        end
    end;
end;
hooksecurefunc("TargetFrame_UpdateAuraPositions", UpdateTargetAuraPositions)

-- events
a:SetScript("OnEvent", function(self, event, ...)
    color = nil;
    if event == "PLAYER_ENTERING_WORLD" then
        color = RAID_CLASS_COLORS[select(2, UnitClass("player"))];
        if color then
            PlayerFrameHealthBar:SetStatusBarColor(color.r, color.g, color.b);
            PlayerFrameHealthBar.lockColor = true;
        end
        TargetFrame:ClearAllPoints();
        TargetFrame:SetPoint("LEFT", PlayerFrame, "RIGHT", 100, 0);
        GetHealthValues("player");
        SetUnitframe("player");
        SetUnitframe("target");
        -- RuneFrame needs a special sort of position
        if THEK_UNITFRAMES["RuneFrame"] then
            RuneFrame:ClearAllPoints();
            RuneFrame:SetPoint(THEK_UNITFRAMES["RuneFrame"][1], UIParent, THEK_UNITFRAMES["RuneFrame"][3], THEK_UNITFRAMES["RuneFrame"][4], THEK_UNITFRAMES["RuneFrame"][5]);
        end;
        
        LARGE_AURA_SIZE = c["targetaura"].largeSize;
        SMALL_AURA_SIZE = c["targetaura"].smallSize;
    elseif event == "PLAYER_TARGET_CHANGED" then
        color = RAID_CLASS_COLORS[select(2, UnitClass("target"))];
        if color then
            TargetFrameHealthBar:SetStatusBarColor(color.r, color.g, color.b);
            TargetFrameHealthBar.lockColor = true;
        end
        color = RAID_CLASS_COLORS[select(2, UnitClass("targetoftarget"))];
        if color then
            TargetFrameToTHealthBar:SetStatusBarColor(color.r, color.g, color.b);
            TargetFrameToTHealthBar.lockColor = true;
        end
        GetHealthValues("target");
    elseif event == "PLAYER_FOCUS_CHANGED" then
        color = RAID_CLASS_COLORS[select(2, UnitClass("focus"))];
        if color then
            FocusFrameHealthBar:SetStatusBarColor(color.r, color.g, color.b);
            FocusFrameHealthBar.lockColor = true;
        end
        GetHealthValues("focus");
    elseif event == "UNIT_HEALTH" then
        local unit = ...;
        if c[unit] then GetHealthValues(unit); end;
    elseif event == "PLAYER_REGEN_DISABLED" or event == "PLAYER_REGEN_ENABLED" then
        SetUnitframe("player");
        SetUnitframe("target");
    elseif event == "VARIABLES_LOADED" then
        if not THEK_UNITFRAMES then THEK_UNITFRAMES = {} end;
        self:UnregisterEvent(event);
    end;
end)

a:RegisterEvent("UNIT_HEALTH");
a:RegisterEvent("PLAYER_ENTERING_WORLD");
a:RegisterEvent("PLAYER_TARGET_CHANGED");
a:RegisterEvent("PLAYER_FOCUS_CHANGED");
a:RegisterEvent("PLAYER_REGEN_DISABLED");
a:RegisterEvent("PLAYER_REGEN_ENABLED");
a:RegisterEvent("VARIABLES_LOADED");

-- slash command
SLASH_thek_Unitframes1 = "/tuf";
SlashCmdList["thek_Unitframes"] = function()
	for v, _ in pairs(uf) do
        if v ~= "focus" then
            a[v].locked = not a[v].locked;
			if a[v].locked then
				a[v]:EnableMouse(false);
				a[v].name:SetText("");
                a[v]:SetBackdrop({bgFile = ""});
			else
				a[v]:EnableMouse(true);
				a[v].name:SetText(v);
                a[v]:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background"});
                a[v]:SetBackdropColor(0, 0, 0, .55);
			end;
        end;
	end
end